@@ -4,18 +4,20 @@ oauthlib.oauth2.rfc6749.grant_types
 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 """
 from __future__ import unicode_literals, absolute_import
-import json
+
+import logging
 
 from oauthlib import common
-from oauthlib.common import log
 from oauthlib.uri_validate import is_absolute_uri
 
 from .base import GrantTypeBase
 from .. import errors
-from ..request_validator import RequestValidator
+
+log = logging.getLogger(__name__)
 
 
 class AuthorizationCodeGrant(GrantTypeBase):
+
     """`Authorization Code Grant`_
 
     The authorization code grant type is used to obtain both access
@@ -90,8 +92,10 @@ class AuthorizationCodeGrant(GrantTypeBase):
 
     .. _`Authorization Code Grant`: http://tools.ietf.org/html/rfc6749#section-4.1
     """
-    def __init__(self, request_validator=None):
-        self.request_validator = request_validator or RequestValidator()
+    grant_types = ['authorization_code']
+    mandatory_parameters = ['grant_type', 'code']
+    disallowed_duplicates = ['grant_type', 'code', 'client_id', 'redirect_uri', 'scope']
+    refresh_token = True
 
     def create_authorization_code(self, request):
         """Generates an authorization grant represented as a dictionary."""
@@ -131,7 +135,7 @@ class AuthorizationCodeGrant(GrantTypeBase):
         :param request: oauthlib.commong.Request
         :param token_handler: A token handler instace, for example of type
                               oauthlib.oauth2.BearerToken.
-        :returns: uri, headers, body, status
+        :returns: headers, body, status
         :raises: FatalClientError on invalid redirect URI or client id.
                  ValueError if scopes are not set on the request object.
 
@@ -203,38 +207,13 @@ class AuthorizationCodeGrant(GrantTypeBase):
         except errors.OAuth2Error as e:
             log.debug('Client error during validation of %r. %r.', request, e)
             request.redirect_uri = request.redirect_uri or self.error_uri
-            return common.add_params_to_uri(request.redirect_uri, e.twotuples), None, None, e.status_code
+            return {'Location': common.add_params_to_uri(request.redirect_uri, e.twotuples)}, None, 302
 
         grant = self.create_authorization_code(request)
         log.debug('Saving grant %r for %r.', grant, request)
-        self.request_validator.save_authorization_code(request.client_id, grant, request)
-        return common.add_params_to_uri(request.redirect_uri, grant.items()), None, None, 302
-
-    def create_token_response(self, request, token_handler):
-        """Validate the authorization code.
-
-        The client MUST NOT use the authorization code more than once. If an
-        authorization code is used more than once, the authorization server
-        MUST deny the request and SHOULD revoke (when possible) all tokens
-        previously issued based on that authorization code. The authorization
-        code is bound to the client identifier and redirection URI.
-        """
-        headers = {
-                'Content-Type': 'application/json;charset=UTF-8',
-                'Cache-Control': 'no-store',
-                'Pragma': 'no-cache',
-        }
-        try:
-            self.validate_token_request(request)
-            log.debug('Token request validation ok for %r.', request)
-        except errors.OAuth2Error as e:
-            log.debug('Client error during validation of %r. %r.', request, e)
-            return None, headers, e.json, e.status_code
-
-        token = token_handler.create_token(request, refresh_token=True)
-        self.request_validator.invalidate_authorization_code(
-                request.client_id, request.code, request)
-        return None, headers, json.dumps(token), 200
+        self.request_validator.save_authorization_code(
+            request.client_id, grant, request)
+        return {'Location': common.add_params_to_uri(request.redirect_uri, grant.items())}, None, 302
 
     def validate_authorization_request(self, request):
         """Check the authorization request for normal and fatal errors.
@@ -261,10 +240,11 @@ class AuthorizationCodeGrant(GrantTypeBase):
         # REQUIRED. The client identifier as described in Section 2.2.
         # http://tools.ietf.org/html/rfc6749#section-2.2
         if not request.client_id:
-            raise errors.MissingClientIdError(state=request.state, request=request)
+            raise errors.MissingClientIdError(request=request)
 
-        if not self.request_validator.validate_client_id(request.client_id, request):
-            raise errors.InvalidClientIdError(state=request.state, request=request)
+        if not self.request_validator.validate_client_id(
+                request.client_id, request):
+            raise errors.InvalidClientIdError(request=request)
 
         # OPTIONAL. As described in Section 3.1.2.
         # http://tools.ietf.org/html/rfc6749#section-3.1.2
@@ -274,18 +254,18 @@ class AuthorizationCodeGrant(GrantTypeBase):
             request.using_default_redirect_uri = False
             log.debug('Using provided redirect_uri %s', request.redirect_uri)
             if not is_absolute_uri(request.redirect_uri):
-                raise errors.InvalidRedirectURIError(state=request.state, request=request)
+                raise errors.InvalidRedirectURIError(request=request)
 
             if not self.request_validator.validate_redirect_uri(
                     request.client_id, request.redirect_uri, request):
-                raise errors.MismatchingRedirectURIError(state=request.state, request=request)
+                raise errors.MismatchingRedirectURIError(request=request)
         else:
             request.redirect_uri = self.request_validator.get_default_redirect_uri(
-                    request.client_id, request)
+                request.client_id, request)
             request.using_default_redirect_uri = True
             log.debug('Using default redirect_uri %s.', request.redirect_uri)
             if not request.redirect_uri:
-                raise errors.MissingRedirectURIError(state=request.state, request=request)
+                raise errors.MissingRedirectURIError(request=request)
 
         # Then check for normal errors.
 
@@ -299,76 +279,71 @@ class AuthorizationCodeGrant(GrantTypeBase):
         # Note that the correct parameters to be added are automatically
         # populated through the use of specific exceptions.
         if request.response_type is None:
-            raise errors.InvalidRequestError(state=request.state,
-                    description='Missing response_type parameter.', request=request)
+            raise errors.InvalidRequestError(
+                description='Missing response_type parameter.',
+                request=request)
 
-        for param in ('client_id', 'response_type', 'redirect_uri', 'scope', 'state'):
+        for param in ('client_id', 'response_type', 'redirect_uri', 'scope',
+                      'state'):
             if param in request.duplicate_params:
-                raise errors.InvalidRequestError(state=request.state,
-                        description='Duplicate %s parameter.' % param, request=request)
+                raise errors.InvalidRequestError(
+                    description='Duplicate %s parameter.' % param,
+                    request=request)
 
-        if not self.request_validator.validate_response_type(request.client_id,
-                request.response_type, request.client, request):
+        if not self.request_validator.validate_response_type(
+                request.client_id, request.response_type, request.client,
+                request):
             log.debug('Client %s is not authorized to use response_type %s.',
                       request.client_id, request.response_type)
             raise errors.UnauthorizedClientError(request=request)
 
         # REQUIRED. Value MUST be set to "code".
         if request.response_type != 'code':
-            raise errors.UnsupportedResponseTypeError(state=request.state, request=request)
+            raise errors.UnsupportedResponseTypeError(request=request)
 
         # OPTIONAL. The scope of the access request as described by Section 3.3
         # http://tools.ietf.org/html/rfc6749#section-3.3
         self.validate_scopes(request)
 
         return request.scopes, {
-                'client_id': request.client_id,
-                'redirect_uri': request.redirect_uri,
-                'response_type': request.response_type,
-                'state': request.state,
+            'client_id': request.client_id,
+            'redirect_uri': request.redirect_uri,
+            'response_type': request.response_type,
+            'state': request.state,
+            'request': request,
         }
 
     def validate_token_request(self, request):
-        # REQUIRED. Value MUST be set to "authorization_code".
-        if request.grant_type != 'authorization_code':
-            raise errors.UnsupportedGrantTypeError(request=request)
+        self.early_validate_token_request(request)
 
-        if request.code is None:
-            raise errors.InvalidRequestError(
-                    description='Missing code parameter.', request=request)
-
-        for param in ('client_id', 'grant_type', 'redirect_uri'):
-            if param in request.duplicate_params:
-                raise errors.InvalidRequestError(state=request.state,
-                        description='Duplicate %s parameter.' % param,
-                        request=request)
-
-        # If the client type is confidential or the client was issued client
-        # credentials (or assigned other authentication requirements), the
-        # client MUST authenticate with the authorization server as described
-        # in Section 3.2.1.
-        # http://tools.ietf.org/html/rfc6749#section-3.2.1
-        if not self.request_validator.authenticate_client(request):
+        if self.request_validator.client_authentication_required(request):
+            # If the client type is confidential or the client was issued
+            # client credentials (or assigned other authentication
+            # requirements), the client MUST authenticate with the
+            # authorization server as described in Section 3.2.1.
+            # http://tools.ietf.org/html/rfc6749#section-3.2.1
+            if not self.request_validator.authenticate_client(request):
+                log.debug('Client authentication failed, %r.', request)
+                raise errors.InvalidClientError(request=request)
+        elif not self.request_validator.authenticate_client_id(
+                request.client_id, request):
             # REQUIRED, if the client is not authenticating with the
             # authorization server as described in Section 3.2.1.
             # http://tools.ietf.org/html/rfc6749#section-3.2.1
-            if not self.request_validator.authenticate_client_id(
-                    request.client_id, request):
-                log.debug('Client authentication failed, %r.', request)
-                raise errors.InvalidClientError(request=request)
-        else:
-            if not hasattr(request.client, 'client_id'):
-                raise NotImplementedError('Authenticate client must set the '
-                                          'request.client.client_id attribute '
-                                          'in authenticate_client.')
+            log.debug('Client authentication failed, %r.', request)
+            raise errors.InvalidClientError(request=request)
+
+        if not hasattr(request.client, 'client_id'):
+            raise NotImplementedError('Authenticate client must set the '
+                                      'request.client.client_id attribute '
+                                      'in authenticate_client.')
 
-        # Ensure client is authorized use of this grant type
         self.validate_grant_type(request)
 
         # REQUIRED. The authorization code received from the
         # authorization server.
-        if not self.request_validator.validate_code(request.client_id,
-                request.code, request.client, request):
+        if not self.request_validator.validate_code(
+                request.client_id, request.code, request.client, request):
             log.debug('Client, %r (%r), is not allowed access to scopes %r.',
                       request.client_id, request.client, request.scopes)
             raise errors.InvalidGrantError(request=request)
@@ -380,8 +355,9 @@ class AuthorizationCodeGrant(GrantTypeBase):
         # REQUIRED, if the "redirect_uri" parameter was included in the
         # authorization request as described in Section 4.1.1, and their
         # values MUST be identical.
-        if not self.request_validator.confirm_redirect_uri(request.client_id,
-                request.code, request.redirect_uri, request.client):
+        if not self.request_validator.confirm_redirect_uri(
+                request.client_id, request.code, request.redirect_uri,
+                request.client):
             log.debug('Redirect_uri (%r) invalid for client %r (%r).',
                       request.redirect_uri, request.client_id, request.client)
             raise errors.AccessDeniedError(request=request)
